#pragma once

// This file contains macros useful when writing code to be compiled both on 68K and on PPC.
// Author: Franois Pottier (pottier@clipper.ens.fr)
// Release: November 2, 1995

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// Macro to determine whether a system routine is available. Returns true on 68K. This is useful when using weak-linked libraries.

#if USESROUTINEDESCRIPTORS
#define	RoutineIsAvailable(name)						true
#else
#define	RoutineIsAvailable(name)						((long) &name != kUnresolvedCFragSymbolAddress)
#endif

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// Apple has defined macros to create and use either a Routine Descriptor or a ProcPtr transparently. The problem is, these macros allocate descriptors from the heap; allocating them
// from global storage, would be cooler. The BUILD_ROUTINE_DESCRIPTOR macro does the job, but works only in PPC mode. The macros below work in both modes.
// They were designed by Zalman Stern.

// MakeStaticRoutineDesc(name, procinfo) creates a static descriptor on PPC and does nothing on 68K.
// ExternRoutineDescriptor(name) declares an extern descriptor on PPC and does nothing on 68K.
// ConditionalRD(name) returns a pointer to the routine descriptor on PPC and a procedure pointer on 68K.

#if GENERATINGCFM
#define	MakeStaticRoutineDesc(name, procinfo) 			static RoutineDescriptor name##RDS = BUILD_ROUTINE_DESCRIPTOR(procinfo, name);
#else
#define	MakeStaticRoutineDesc(name, procinfo)
#endif

#if GENERATINGCFM
#define	ExternRoutineDescriptor(name) 				extern RoutineDescriptor name##RDS;
#else
#define	ExternRoutineDesc(name)
#endif

#if GENERATINGCFM
#define	ConditionalRD(name)							(&name##RDS)
#else
#define	ConditionalRD(name)							(&name)
#endif

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
