//	----------------------------------------------------------------------------
//	MemoryAllocator.h
//	----------------------------------------------------------------------------
//	By Jean-Franois Brouillet, Copyright  1995
//
//	V0.001		951023	JFB		Use of dynarray class to ease error recovery
//								when exceptions are thrown.
//	V0.000		951017	JFB		Creation
//
//	No restriction on use. But please, use with caution as this is not
//	thoroughly tested. Also, understand when the use of such an allocator is
//	appropriate for your program : speed doesn't come for free !
//
//	The whole idea of this allocator is to manage memory much more quickly than
//	what standard operator new / delete provide. For this to happen we sacrifice
//	some features;
//	Our speed improvment comes from the fact we're linking freed objects into
//	a list where new tries first to draw from. Only if this list is empty does
//	new try the standard new operator.
//	Average improvments range from 5x to 10x the speed...but memory held back
//	to one pool cand not be used by another !
//	A dynamic list of pools is created on the fly.

#ifndef		__MEMORYALLOCATOR__
#define		__MEMORYALLOCATOR__

#include	<stdlib.h>
#include	"dynarray.t"

//	----------------------------------------------------------------------------
//	MemoryAllocator
//	----------------------------------------------------------------------------
class	MemoryAllocator
{
public:
							MemoryAllocator();
virtual						~MemoryAllocator();

virtual	void *				allocate(size_t inBytes);
virtual	void				release(void * inObject);

virtual	void				releaseAll(void);

static	MemoryAllocator *	GetAllocator(void);

protected:

struct	heapNode
{							//	------------------------------------------------
	size_t		size;		//	First 4 bytes = size of object,
	heapNode *	next;		//	anything else is part of the new'd object
};							//	or free for storing a link when recycled
							//	------------------------------------------------

virtual	heapNode * &		getHeapHeader(size_t inObjectSize);

		dynarray<size_t>	m_ObjectSizeArray;
		dynarray<heapNode *>
							m_ObjectArray;

static	MemoryAllocator *	M_Singleton;
};

#endif	/*	__MEMORYALLOCATOR__	*/