//	----------------------------------------------------------------------------
//	TestAllocator.cp
//	----------------------------------------------------------------------------
//	By Jean-Franois Brouillet, Copyright  1995
//
//	V0.000		951012	JFB		Creation
//
//	No restriction on use. But please, use with caution as this is not
//	thoroughly tested. Also, understand when the use of such an allocator is
//	appropriate for your program : speed doesn't come for free !

#include	<new>
#include	<exception>
#include	<stdio.h>
#include	<string.h>

#include	"Accessors.t"
#include	"MemoryAllocator.h"

#include	<Profiler.h>

//const	size_t	kArraySize	=	10002;
//const	size_t	kArraySize	=	100002;
const	size_t	kArraySize	=	10002;
//const	size_t	kArraySize	=	102;

class	C1;
class	C2;
class	C3;

C1 **	c1array;
C2 **	c2array;
C3 **	c3array;

long	gStartTime;
long	gEndTime;

inline	long	getTicks(void)			{	return * (long *) 0x16a;		}	//	hack!
inline	void	startTimer(void)		{	gStartTime = getTicks();		}
inline	void	stopTimer(void)			{	gEndTime = getTicks();			}
inline	long	timerDuration(void)		{	return gEndTime - gStartTime;	}

class	C1
{
public:
				C1()	{
#if	!__profile__
					** (long **) 0x824 ^= -1;	//	hack!	0x824 = ScrnBase => blink upper corner
					size_t	x1	=	sizeof(size_t) + align32bits(sizeof(C1));
					size_t	x2	=	sizeof(long) + sizeof(size_t) + align32bits(sizeof(C1));
					size_t	x3	=	sizeof(long) + sizeof(long) + sizeof(size_t) + align32bits(sizeof(C1));
					char *	raw	=	((char *) this) -4;
					long	reqSz =	* (long *) raw;
					
							if ((x1 != reqSz) && (x2 != reqSz) && (x3 != reqSz))
							{
								DebugStr("\pcorrupted heap in Cx::Cx() !");
							}
#endif
						};
virtual			~C1() {
#if	!__profile__
					** (long **) 0x824 ^= -1;	//	hack!	0x824 = ScrnBase => blink upper corner
					size_t	x1	=	sizeof(size_t) + align32bits(sizeof(C1));
					size_t	x2	=	sizeof(long) + sizeof(size_t) + align32bits(sizeof(C1));
					size_t	x3	=	sizeof(long) + sizeof(long) + sizeof(size_t) + align32bits(sizeof(C1));
					char *	raw	=	((char *) this) -4;
					long	reqSz =	* (long *) raw;
					
							if ((x1 != reqSz) && (x2 != reqSz) && (x3 != reqSz))
							{
								DebugStr("\pcorrupted heap in Cx::~Cx() !");
							}
#endif
					};
long	data1;

		void *	operator new(size_t inSize) 	{	return MemoryAllocator::GetAllocator()->allocate(inSize);	};
		void	operator delete(void * inObj)	{	MemoryAllocator::GetAllocator()->release(inObj);			};
};

class	C2 : public C1
{
public:
				C2() {};
virtual			~C2() {};
long	data2;
};

class	C3 : public C2
{
public:
				C3() {};
virtual			~C3() {};
long	data3;
};

static
void	test1(void)
{
int 	index;
int		loop = 3;
long	newTime[3];
long	delTime[3];

	while (loop--)
	{
		startTimer();
		for(index = 0; index < kArraySize; ++index)
		{
			** (long **) 0x824 ^= -1;

			c1array[index] = new C1;
			c2array[index] = new C2;
			c3array[index] = new C3;
		}
		stopTimer();
		newTime[loop] = timerDuration();

		startTimer();
		for(index = 0; index < kArraySize; ++index)
		{
			** (long **) 0x824 ^= -1;

			delete c1array[index];	c1array[index] = 0;
			delete c2array[index];	c2array[index] = 0;
			delete c3array[index];	c3array[index] = 0;
		}
		stopTimer();
		delTime[loop] = timerDuration();
	}


	printf("\n### alocating %d objects in %ld ticks", kArraySize, newTime[2]);
	printf("\n### freeing %d objects in %ld ticks", kArraySize, delTime[2]);
	printf("\n### reusing %d objects in %ld ticks", kArraySize, newTime[1]);
	printf("\n### re-freeing %d objects in %ld ticks", kArraySize, delTime[1]);
	printf("\n### (same player--play again) reusing %d objects in %ld ticks", kArraySize, newTime[0]);
	printf("\n### (same player--play again) re-freeing %d objects in %ld ticks", kArraySize, delTime[0]);
	printf("\n");
}

void	main(void)
{
	printf("\n### Testing Allocator\n");

   #if __profile__
		OSErr  theErr;
		theErr = ProfilerInit( collectDetailed, bestTimeBase, 100, 20 );
		if (theErr)
			abort();          
   #endif


	c1array = (C1 **) ::operator new (sizeof(C1 *) * kArraySize);
	c2array = (C2 **) ::operator new (sizeof(C2 *) * kArraySize);
	c3array = (C3 **) ::operator new (sizeof(C3 *) * kArraySize);

	try
	{
		test1();
	}
	
	catch(...)
	{
		printf("\n### Uncaught exception raised !!!!!\n");
	}

   #if __profile__
       ProfilerDump("\pTestAllocator.profiler");
       ProfilerTerm();
   #endif
}
